/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <gnome.h>
#include <gtk/gtk.h>
#include <pthread.h>

#include "pan.h"

/* 'File' menu */
static GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_MENU_EXIT_ITEM(pan_shutdown, NULL),
	GNOMEUIINFO_END
};

/* 'Edit' menu */
static GnomeUIInfo edit_menu[] = {
	GNOMEUIINFO_MENU_CUT_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_COPY_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_PASTE_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_SELECT_ALL_ITEM (not_implemented, NULL),
	GNOMEUIINFO_MENU_CLEAR_ITEM (not_implemented, NULL),
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Find"), "Find text in displayed header fields", articlelist_find_text_cb,
	 NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'F', GDK_CONTROL_MASK, NULL},
	{GNOME_APP_UI_ITEM, _("Find Next"), "Repeat the last search", articlelist_find_next_cb,
	 NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'G', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

/* 'Settings' menu */
static GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(props_spawn, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo servers_menu[] = {
	GNOMEUIINFO_END
};

/* 'Online' menu */
static GnomeUIInfo online_menu[] = {
	{GNOME_APP_UI_ITEM, _("Go Online"), "Go online", go_online, 0},
	{GNOME_APP_UI_ITEM, _("Go Offline"), "Go online", go_offline_mf},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Stop all Online Tasks"), "Stop all online tasks", stop_all_mf},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Get all Groups"), "Get all groups", get_all_groups},
	GNOMEUIINFO_END
};

/* 'Help' menu */
static GnomeUIInfo help_menu[] = {
	GNOMEUIINFO_ITEM_STOCK (_("About..."), NULL,
				gui_about, GNOME_STOCK_MENU_ABOUT),
	GNOMEUIINFO_END
};

/* 'Windows' menu */
static GnomeUIInfo windows_menu[] = {
	{GNOME_APP_UI_ITEM, _("Notebook View"), "Switch to Notebook View", gui_construct_notebook},
	{GNOME_APP_UI_ITEM, _("Paned View"), "Switch to Paned View", gui_construct_paned},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Log Viewer"), "Open log viewer", not_implemented},
	{GNOME_APP_UI_ITEM, _("Queue Manager"), "Open Queue manager", not_implemented},
	GNOMEUIINFO_END
};

/* 'Group' menu */
static GnomeUIInfo group_menu[] = {
	{GNOME_APP_UI_ITEM, _("Get All Article Headers"), "Get all article headers", download_selected_groups_cb},
	{GNOME_APP_UI_ITEM, _("Get New Article Headers"), "Get new article headers", download_selected_groups_new_cb},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Properties"), "Set group properties", not_implemented},
	GNOMEUIINFO_END
};

/* 'Message' menu */
static GnomeUIInfo message_menu[] = {
	{GNOME_APP_UI_ITEM, _("Post New Message..."), "Post a new message", post_window},
	{GNOME_APP_UI_ITEM, _("Post Followup Message..."), "Post a followup message", followup_window},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Decode Message"), "Decode messages", decode_selected_articles,
	 NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL},
	{GNOME_APP_UI_ITEM, _("Delete Message"), "Delete message from database", article_delete},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Select All"), "Select all messages", articlelist_select_all,
	 NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'A', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
	GNOMEUIINFO_SUBTREE(_("Windows"), &windows_menu),
	GNOMEUIINFO_SUBTREE(_("Servers"), &servers_menu),
	GNOMEUIINFO_SUBTREE(_("Online"), &online_menu),
	GNOMEUIINFO_SUBTREE(_("Group"), &group_menu),
	GNOMEUIINFO_SUBTREE(_("Message"), &message_menu),
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END
};

static GnomeUIInfo articlelist_menu_popup[] = {
	{GNOME_APP_UI_ITEM, "Decode Message", "Decode messages", decode_selected_articles,
	NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'D', GDK_CONTROL_MASK, NULL },
	{GNOME_APP_UI_ITEM, _("Delete Message"), "Delete message from database", article_delete},
	GNOMEUIINFO_END
};

static GnomeUIInfo groups_menu_popup[] = {
	{GNOME_APP_UI_ITEM, _("Subscribe to Selected"), "subscribe to newsgroup", subscribe_to_selected_cb},
	{GNOME_APP_UI_ITEM, _("Unsubscribe from Selected"), "unsubscribe", unsubscribe_from_selected_cb},
	GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM, _("Get All Article Headers"), "Get all article headers", download_selected_groups_cb},
	{GNOME_APP_UI_ITEM, _("Get New Article Headers"), "Get new article headers", download_selected_groups_new_cb},
	GNOMEUIINFO_END
};

/* Global Variables */
static GtkWidget *appbar;
static GtkWidget *progress;

static GtkWidget *groups_vbox;
static GtkWidget *articlelist_ctree;
GtkWidget *text_table;
static GtkWidget *contents_vbox;

int viewmode = 0;		/* 1 = paned, 2 = notebook */
GtkWidget *notebook;
extern GtkWidget *text;
extern article_data *current_article;
extern int group_mode;

GtkWidget *
articlelist_menu_get_popup (void)
{
	static GtkWidget *articlelist_popup = NULL;

	if (articlelist_popup) return articlelist_popup;

	articlelist_popup = gnome_popup_menu_new (articlelist_menu_popup);
	return articlelist_popup;
}

GtkWidget *
group_clist_menu_get_popup (void)
{
	static GtkWidget *groups_popup = NULL;
	
	if (groups_popup) return groups_popup;

	groups_popup = gnome_popup_menu_new (groups_menu_popup);
	return groups_popup;
	
}

int gui_get_viewmode(void)
{
	return viewmode;
}

void gui_clean(void)
{
	pan_lock();
	gnome_appbar_clear_stack(GNOME_APPBAR(appbar));
	if (progress) {
		gtk_progress_set_activity_mode(GTK_PROGRESS(progress), FALSE);
		progress = NULL;
	}
	gnome_appbar_set_progress(GNOME_APPBAR(appbar), 0);
	pan_unlock();
}


void gui_create_appbar(GtkWidget * window)
{
	appbar = gnome_appbar_new(TRUE, TRUE, FALSE);
	gnome_appbar_set_default(GNOME_APPBAR(appbar), _("Pan - Offline"));
	gnome_app_set_statusbar(GNOME_APP(window), appbar);
}


void gui_set_status(gchar * msg)
{
	pan_lock();
	gnome_appbar_pop(GNOME_APPBAR(appbar));
	gnome_appbar_push(GNOME_APPBAR(appbar), msg);
	pan_unlock();
}


void gui_set_status_va(const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail(format != NULL);

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);

	pan_lock();
	gnome_appbar_pop(GNOME_APPBAR(appbar));
	gnome_appbar_push(GNOME_APPBAR(appbar), msg);
	pan_unlock();
	g_free(msg);
}


void gui_set_progress(gfloat progress)
{
	pan_lock();
	if ((progress >= 0) && (progress <= 1))
		gnome_appbar_set_progress(GNOME_APPBAR(appbar), progress);
	pan_unlock();
}


void gui_set_status_default(gchar * msg)
{
	pan_lock();
	gnome_appbar_set_default(GNOME_APPBAR(appbar), msg);
	pan_unlock();
}


void gui_set_progress_activity(gint count)
{
	pan_lock();
	if (progress == NULL) {
		progress =
			GTK_WIDGET(gnome_appbar_get_progress(GNOME_APPBAR(appbar)));
		gtk_progress_set_activity_mode(GTK_PROGRESS(progress), TRUE);
		gtk_progress_bar_set_discrete_blocks(GTK_PROGRESS_BAR(progress),
						     10);
		gtk_progress_bar_set_activity_step(GTK_PROGRESS_BAR(progress), 1);
	}
	gtk_progress_set_value(GTK_PROGRESS(progress), (gfloat) (count % 11));
	pan_unlock();
}

/* Infobar */
GtkWidget *infobar;
guint context_id;

GtkWidget *gui_create_infobar(void)
{
	infobar = gtk_statusbar_new();
	context_id =
		gtk_statusbar_get_context_id(GTK_STATUSBAR(infobar), "blah");
	return infobar;
}

void gui_set_info(gchar * msg)
{
	pan_lock();
	gtk_statusbar_pop(GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push(GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock();
}

void gui_set_info_va(const gchar * format, ...)
{
	va_list args;
	gchar *msg;

	g_return_if_fail(format != NULL);

	va_start(args, format);
	msg = g_strdup_vprintf(format, args);
	va_end(args);

	pan_lock();
	gtk_statusbar_pop(GTK_STATUSBAR(infobar), context_id);
	gtk_statusbar_push(GTK_STATUSBAR(infobar), context_id, msg);
	pan_unlock();
	g_free(msg);
}


/*---[ gui_about ]----------------------------------------------------
 * informative about window popup
 *--------------------------------------------------------------------*/
void gui_about (void)
{
	const gchar *authors[] = {
		"Matt Eagleson <e.Messiah@superpimp.org>",
		"Jason Leach <leach@wam.umd.edu>",
		NULL
	};

	GtkWidget *about = gnome_about_new ("Pan", VERSION,
					    _("Copyright e.Messiah (C) 1999"),
					    authors,
					    "The Pimp Ass Newsreader (PAN)",
					    NULL);
	gtk_widget_show(about);
}

 
/*---[ gui_construct ]------------------------------------------------
 * initialize and place the basic interface components
 *--------------------------------------------------------------------*/
void
gui_construct (void)
{
	GtkWidget *w, *x;
	gint geo_x=0, geo_y=0, width=640, height=400;

	Pan.window = gnome_app_new ("Pan", _("Pan"));
	gtk_signal_connect (GTK_OBJECT(Pan.window), "delete_event", 
			    GTK_SIGNAL_FUNC (pan_shutdown), NULL);

	width = gnome_config_get_int ("/Pan/Geometry/width=640");
	height = gnome_config_get_int ("/Pan/Geometry/height=400");

	gtk_window_set_default_size (GTK_WINDOW(Pan.window), width, height);
/*	gtk_widget_set_usize (GTK_WIDGET(Pan.window), 640, 400); */

	gnome_app_create_menus (GNOME_APP(Pan.window), main_menu);

/* Groups vbox */
	groups_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_set_usize (GTK_WIDGET(groups_vbox), 300, 200);
	x = gtk_hbox_new (FALSE, 0);
	w = gtk_label_new ("Find:");
	gtk_box_pack_start (GTK_BOX (x), w, FALSE, FALSE, 3);
	w = gtk_entry_new ();
	gtk_signal_connect (GTK_OBJECT (w), "activate",
			    GTK_SIGNAL_FUNC(grouplist_grep), w);
	gtk_box_pack_start (GTK_BOX (x), w, TRUE, TRUE, 3);

	gtk_box_pack_start (GTK_BOX (groups_vbox), grouplist_create(), TRUE, TRUE, 1);
	gtk_box_pack_start (GTK_BOX (groups_vbox), x, FALSE, FALSE, 1);

/* Articlelist_ctree */
	w = create_articlelist_ctree ();
	articlelist_ctree = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (articlelist_ctree), w, TRUE, TRUE, 0);

/* Text table */
	w = text_create ();
	text_table = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (text_table), w, TRUE, TRUE, 0);

	gui_create_appbar (Pan.window);

	contents_vbox = gtk_vbox_new (FALSE, 0);
	gnome_app_set_contents (GNOME_APP (Pan.window), contents_vbox);

	if (gnome_config_get_int("Pan/State/viewmode=1") == 2)
		gui_construct_notebook();
	else
		gui_construct_paned();

	/* FIXME: 0,0 shows up under the panel for me on IceWm
	 * I'm not sure we should mess with window placement
	 * until the user tells us to... 
	 geo_x = gnome_config_get_int ("/Pan/Geometry/pan_x=0");
	 geo_y = gnome_config_get_int ("/Pan/geometry/pan_y=0");
	 gdk_window_move (Pan.window->window, geo_x, geo_y);
	*/
}

/*---[ gui_construct_paned ]------------------------------------------
 * build the paned GUI layout
 *--------------------------------------------------------------------*/
void
gui_construct_paned (void)
{
	GtkWidget *vpaned;
	GtkWidget *hpaned;
	GtkWidget *w;

	if (viewmode == GUI_PANED) return;
	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_ref (groups_vbox);
		gtk_container_remove (GTK_CONTAINER (groups_vbox->parent), groups_vbox);
		gtk_widget_ref (articlelist_ctree);
		gtk_container_remove (GTK_CONTAINER (articlelist_ctree->parent), articlelist_ctree);
		gtk_widget_ref (text_table);
		gtk_container_remove (GTK_CONTAINER (text_table->parent), text_table);
		gtk_container_foreach (GTK_CONTAINER (contents_vbox), (GtkCallback) gtk_widget_destroy, NULL);
	}

	vpaned = gtk_vpaned_new ();
	hpaned = gtk_hpaned_new ();
	gtk_paned_add1 (GTK_PANED(hpaned), groups_vbox);
	gtk_paned_add2 (GTK_PANED(hpaned), articlelist_ctree);
	gtk_paned_add1 (GTK_PANED(vpaned), hpaned);
	gtk_paned_add2 (GTK_PANED(vpaned), text_table);
	gtk_box_pack_start (GTK_BOX (contents_vbox), vpaned, TRUE, TRUE, 0);

	gui_create_infobar ();
	gtk_box_pack_start (GTK_BOX (contents_vbox), infobar, FALSE, FALSE, 0);

	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_unref (groups_vbox);
		gtk_widget_unref (articlelist_ctree);
		gtk_widget_unref (text_table);
	}

	viewmode = GUI_PANED;
	gtk_widget_show_all(GTK_WIDGET(vpaned));
}


/*---[ gui_construct_notebook ]---------------------------------------
 * build and pack the notebook GUI style layout
 *--------------------------------------------------------------------*/
void
gui_construct_notebook (void)
{
	GtkWidget *w;

	if (viewmode == GUI_NOTEBOOK) return;
	if (viewmode == GUI_PANED)
	{
		gtk_widget_ref (groups_vbox);
		gtk_container_remove (GTK_CONTAINER (groups_vbox->parent), groups_vbox);
		gtk_widget_ref (articlelist_ctree);
		gtk_container_remove (GTK_CONTAINER (articlelist_ctree->parent), articlelist_ctree);
		gtk_widget_ref (text_table);
		gtk_container_remove (GTK_CONTAINER (text_table->parent), text_table);
		gtk_container_foreach (GTK_CONTAINER (contents_vbox), (GtkCallback) gtk_widget_destroy, NULL);
	}

	notebook = gtk_notebook_new ();
	w = gtk_label_new ("Groups");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), groups_vbox, w);
	w = gtk_label_new ("Articles");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), articlelist_ctree, w);
	w = gtk_label_new ("Message");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), text_table, w);
	gtk_box_pack_start (GTK_BOX (contents_vbox), notebook, TRUE, TRUE, 0);

	gui_create_infobar();
	gtk_box_pack_start (GTK_BOX (contents_vbox), infobar, FALSE, FALSE, 0);

	gtk_notebook_set_tab_border (GTK_NOTEBOOK (notebook), 2);

/* connect signals */
	gtk_signal_connect (GTK_OBJECT (Pan.article_ctree), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);
	gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb),  NULL);
	gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);

	if (viewmode == GUI_PANED)
	{
		gtk_widget_unref (groups_vbox);
		gtk_widget_unref (articlelist_ctree);
		gtk_widget_unref (text_table);
	}

	viewmode = GUI_NOTEBOOK;
	gtk_widget_show_all (GTK_WIDGET(notebook));
}


/*--------------------------------------------------------------------
 * flip to a different notebook page, setting keyboard focus on a 
 * widget in the new page
 *--------------------------------------------------------------------*/
void
gui_page_set (gint page, GtkWidget *focus_item)
{
	if (gui_get_viewmode() == GUI_NOTEBOOK) {
		gtk_notebook_set_page (GTK_NOTEBOOK (notebook), page);
		gtk_widget_grab_focus (focus_item);
	}
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
gui_page_change (gint change, GtkWidget *widget)
{
	gint cur_page;
	article_data *adata;
	group_data *gdata;
	gint row;
	GtkWidget *tab;

	if (gui_get_viewmode() != GUI_NOTEBOOK)
		return;

	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	cur_page += change;
	if ((cur_page < 0) || (cur_page > 2)) 
		return;

	if (change < 0) { /* Escape was pressed */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
			gtk_widget_grab_focus (Pan.article_ctree);
			break;
		case MESSAGE_PAGE:
			gtk_widget_grab_focus (text);
			break;
		default:
			break;
		}
	}
	else { /* enter key hit */
/* FIXME: segfaults if selection is zero, but when user hits escape, it
   automatically sends a "undo_selection" signal, deselecting all */
		switch (cur_page) {
		case GROUPS_PAGE:
			gtk_widget_grab_focus (Pan.group_clist);
			break;
		case ARTICLELIST_PAGE:
//			if (!GTK_CLIST (Pan.group_clist)->selection && !GTK_CLIST (Pan.group_clist)->undo_selection)
//				return;

			gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), 
								       GPOINTER_TO_INT (GTK_CLIST (Pan.group_clist)->selection->data));
			if (gdata) {
				if (Pan.active_group == gdata) {
					gtk_widget_grab_focus (Pan.article_ctree);
					break;
				}
				if (group_index_exists(gdata))
					queue_add_articlelist_download (gdata,
									QUEUE_ARTICLELIST_LOAD |
									QUEUE_FOREGROUND);
				else
					choose_download_dialog (gdata);
			}

			gtk_widget_grab_focus (Pan.article_ctree);
			break;
		case MESSAGE_PAGE:
			adata = (article_data *) gtk_ctree_node_get_row_data (GTK_CTREE (Pan.article_ctree), 
									      GTK_CLIST (Pan.article_ctree)->selection->data);
			if (adata) {
				current_article = adata;
				if (adata->body == NULL) {
					queue_add_article_download (adata,
								    QUEUE_ARTICLE_DOWNLOAD |
								    QUEUE_FOREGROUND);
					adata->state |= STATE_DOWNLOAD_QUEUED;
					articlelist_update_node (adata);
				}
				else
					text_update (adata);
			}
			gtk_widget_grab_focus (text);
			break;
		default:
			break;
		}
	}
	gtk_notebook_set_page (GTK_NOTEBOOK (notebook), cur_page);
/*
FIXME: this would be nice, there is a slight problem with the tabs at the top
being refreshed after repeated flipping around.  But redrawing the entire
notebook looks flickery and will waste a lot of resources.  Maybe redraw at
some other less frequent place or redraw only the tab elements (if that is 
possible).
	gtk_widget_draw (GTK_WIDGET (notebook), NULL);
*/
}

void
gui_key_press_cb (GtkWidget* widget, GdkEventKey *event, gpointer data)
{
	switch (event->keyval)
	{
	case GDK_q:
		gui_page_change (-1, widget);
		break;
	case GDK_Return:
		gui_page_change (1, widget);
		break;
	default:
		break;
	}
}


/*---[ gui_shutdown ]-------------------------------------------------
 * save geometry states into config files
 *--------------------------------------------------------------------*/
void
gui_shutdown (void)
{
	gint x=0, y=0, width=640, height=400;

	gdk_window_get_position (Pan.window->window, &x, &y);
	gdk_window_get_size (Pan.window->window, &width, &height);
				
	gnome_config_set_int ("/Pan/Geometry/pan_x", x);
	gnome_config_set_int ("/Pan/Geometry/pan_y", y);
	gnome_config_set_int ("/Pan/Geometry/width", width);
	gnome_config_set_int ("/Pan/Geometry/height", height);
	gnome_config_set_int ("/Pan/State/viewmode", viewmode);
	gnome_config_set_int ("/Pan/State/group_mode", group_mode);
	gnome_config_sync();
}
